package com.sun.imageio.plugins.jpeg;

import org.w3c.dom.NamedNodeMap;
import javax.imageio.IIOException;
import javax.imageio.stream.ImageOutputStream;
import javax.imageio.metadata.IIOMetadataNode;
import java.util.Iterator;
import org.w3c.dom.NodeList;
import javax.imageio.metadata.IIOInvalidTreeException;
import org.w3c.dom.Node;
import javax.imageio.plugins.jpeg.JPEGQTable;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

class DQTMarkerSegment extends MarkerSegment
{
    List tables;
    
    DQTMarkerSegment(float n, boolean b) {
        super(219);
        (this.tables = new ArrayList()).add(new DQTMarkerSegment.Qtable(true, n));
        if (b) {
            this.tables.add(new DQTMarkerSegment.Qtable(false, n));
        }
    }
    
    DQTMarkerSegment(JPEGBuffer jpegBuffer) throws IOException {
        super(jpegBuffer);
        this.tables = new ArrayList();
        Object o;
        for (int i = this.length; i > 0; i -= o.data.length + 1) {
            o = new DQTMarkerSegment.Qtable(jpegBuffer);
            this.tables.add(o);
        }
        jpegBuffer.bufAvail -= this.length;
    }
    
    DQTMarkerSegment(JPEGQTable[] array) {
        super(219);
        this.tables = new ArrayList();
        for (int i = 0; i < array.length; ++i) {
            this.tables.add(new DQTMarkerSegment.Qtable(array[i], i));
        }
    }
    
    DQTMarkerSegment(Node offendingNode) throws IIOInvalidTreeException {
        super(219);
        this.tables = new ArrayList();
        NodeList childNodes = offendingNode.getChildNodes();
        int length = childNodes.getLength();
        if (length < 1 || length > 4) {
            throw new IIOInvalidTreeException("Invalid DQT node", offendingNode);
        }
        for (int i = 0; i < length; ++i) {
            this.tables.add(new DQTMarkerSegment.Qtable(childNodes.item(i)));
        }
        return;
    }
    
    @Override
    protected Object clone() {
        DQTMarkerSegment dqtMarkerSegment = (DQTMarkerSegment)super.clone();
        dqtMarkerSegment.tables = new ArrayList(this.tables.size());
        Iterator<Object> iterator = this.tables.iterator();
        while (iterator.hasNext()) {
            dqtMarkerSegment.tables.add(((DQTMarkerSegment.Qtable)iterator.next()).clone());
        }
        return dqtMarkerSegment;
    }
    
    @Override
    IIOMetadataNode getNativeNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("dqt");
        for (int i = 0; i < this.tables.size(); ++i) {
            iioMetadataNode.appendChild(((DQTMarkerSegment.Qtable)this.tables.get(i)).getNativeNode());
        }
        return iioMetadataNode;
    }
    
    @Override
    void write(ImageOutputStream imageOutputStream) throws IOException {
    }
    
    @Override
    void print() {
        this.printTag("DQT");
        System.out.println("Num tables: " + Integer.toString(this.tables.size()));
        for (int i = 0; i < this.tables.size(); ++i) {
            ((DQTMarkerSegment.Qtable)this.tables.get(i)).print();
        }
        System.out.println();
    }
    
    DQTMarkerSegment.Qtable getChromaForLuma(DQTMarkerSegment.Qtable qtable) {
        boolean n = true;
        int n2 = 1;
        while (true) {
            int n3 = n2;
            qtable.getClass();
            if (n3 < 64) {
                if (qtable.data[n2] != qtable.data[n2 - 1]) {
                    n = false;
                    break;
                }
                ++n2;
            }
            else {
                break;
            }
        }
        if (!(n == 0)) {
            DQTMarkerSegment.Qtable qtable2 = (Object)qtable.clone();
            qtable2.tableID = 1;
            return qtable2;
        }
        int n4 = 0;
        int n5 = 1;
        while (true) {
            int n6 = n5;
            qtable.getClass();
            if (n6 < 64) {
                if (qtable.data[n5] > qtable.data[n4]) {
                    n4 = n5;
                }
                ++n5;
            }
            else {
                break;
            }
        }
        return new DQTMarkerSegment.Qtable(JPEGQTable.K2Div2Chrominance.getScaledInstance((float)qtable.data[n4] / (float)JPEGQTable.K1Div2Luminance.getTable()[n4], true), 1);
    }
    
    DQTMarkerSegment.Qtable getQtableFromNode(Node node) throws IIOInvalidTreeException {
        return new DQTMarkerSegment.Qtable(node);
    }
    
    class Qtable implements Cloneable
    {
        int elementPrecision;
        int tableID;
        final int QTABLE_SIZE = 64;
        int[] data;
        private final int[] zigzag;
        
        Qtable(boolean b, float convertToLinearQuality) {
            this.zigzag = new int[] { 0, 1, 5, 6, 14, 15, 27, 28, 2, 4, 7, 13, 16, 26, 29, 42, 3, 8, 12, 17, 25, 30, 41, 43, 9, 11, 18, 24, 31, 40, 44, 53, 10, 19, 23, 32, 39, 45, 52, 54, 20, 22, 33, 38, 46, 51, 55, 60, 21, 34, 37, 47, 50, 56, 59, 61, 35, 36, 48, 49, 57, 58, 62, 63 };
            this.elementPrecision = 0;
            JPEGQTable jpegqTable;
            if (b) {
                this.tableID = 0;
                jpegqTable = JPEGQTable.K1Div2Luminance;
            }
            else {
                this.tableID = 1;
                jpegqTable = JPEGQTable.K2Div2Chrominance;
            }
            if (convertToLinearQuality != 0.75f) {
                convertToLinearQuality = JPEG.convertToLinearQuality(convertToLinearQuality);
                if (b) {
                    jpegqTable = JPEGQTable.K1Luminance.getScaledInstance(convertToLinearQuality, true);
                }
                else {
                    jpegqTable = JPEGQTable.K2Div2Chrominance.getScaledInstance(convertToLinearQuality, true);
                }
            }
            this.data = jpegqTable.getTable();
        }
        
        Qtable(JPEGBuffer jpegBuffer) throws IIOException {
            this.zigzag = new int[] { 0, 1, 5, 6, 14, 15, 27, 28, 2, 4, 7, 13, 16, 26, 29, 42, 3, 8, 12, 17, 25, 30, 41, 43, 9, 11, 18, 24, 31, 40, 44, 53, 10, 19, 23, 32, 39, 45, 52, 54, 20, 22, 33, 38, 46, 51, 55, 60, 21, 34, 37, 47, 50, 56, 59, 61, 35, 36, 48, 49, 57, 58, 62, 63 };
            this.elementPrecision = jpegBuffer.buf[jpegBuffer.bufPtr] >>> 4;
            this.tableID = (jpegBuffer.buf[jpegBuffer.bufPtr++] & 0xF);
            if (this.elementPrecision != 0) {
                throw new IIOException("Unsupported element precision");
            }
            this.data = new int[64];
            for (int i = 0; i < 64; ++i) {
                this.data[i] = (jpegBuffer.buf[jpegBuffer.bufPtr + this.zigzag[i]] & 0xFF);
            }
            jpegBuffer.bufPtr += 64;
            return;
        }
        
        Qtable(JPEGQTable jpegqTable, int tableID) {
            this.zigzag = new int[] { 0, 1, 5, 6, 14, 15, 27, 28, 2, 4, 7, 13, 16, 26, 29, 42, 3, 8, 12, 17, 25, 30, 41, 43, 9, 11, 18, 24, 31, 40, 44, 53, 10, 19, 23, 32, 39, 45, 52, 54, 20, 22, 33, 38, 46, 51, 55, 60, 21, 34, 37, 47, 50, 56, 59, 61, 35, 36, 48, 49, 57, 58, 62, 63 };
            this.elementPrecision = 0;
            this.tableID = tableID;
            this.data = jpegqTable.getTable();
        }
        
        Qtable(Node node) throws IIOInvalidTreeException {
            this.zigzag = new int[] { 0, 1, 5, 6, 14, 15, 27, 28, 2, 4, 7, 13, 16, 26, 29, 42, 3, 8, 12, 17, 25, 30, 41, 43, 9, 11, 18, 24, 31, 40, 44, 53, 10, 19, 23, 32, 39, 45, 52, 54, 20, 22, 33, 38, 46, 51, 55, 60, 21, 34, 37, 47, 50, 56, 59, 61, 35, 36, 48, 49, 57, 58, 62, 63 };
            if (node.getNodeName().equals("dqtable")) {
                NamedNodeMap attributes = node.getAttributes();
                int length = attributes.getLength();
                if (length < 1 || length > 2) {
                    throw new IIOInvalidTreeException("dqtable node must have 1 or 2 attributes", node);
                }
                this.elementPrecision = 0;
                this.tableID = MarkerSegment.getAttributeValue(node, attributes, "qtableId", 0, 3, true);
                if ((node instanceof IIOMetadataNode)) {
                    JPEGQTable jpegqTable = (JPEGQTable)((IIOMetadataNode)node).getUserObject();
                    if (jpegqTable == null) {
                        throw new IIOInvalidTreeException("dqtable node must have user object", node);
                    }
                    this.data = jpegqTable.getTable();
                    return;
                }
                else {
                    throw new IIOInvalidTreeException("dqtable node must have user object", node);
                }
            }
            else {
                throw new IIOInvalidTreeException("Invalid node, expected dqtable", node);
            }
        }
        
        @Override
        protected Object clone() {
            DQTMarkerSegment.Qtable qtable = null;
            try {
                qtable = (DQTMarkerSegment.Qtable)super.clone();
            }
            catch (CloneNotSupportedException ex) {}
            if (this.data != null) {
                qtable.data = (int[])(int[])this.data.clone();
                return qtable;
            }
            return qtable;
        }
        
        IIOMetadataNode getNativeNode() {
            IIOMetadataNode iioMetadataNode = new IIOMetadataNode("dqtable");
            iioMetadataNode.setAttribute("elementPrecision", Integer.toString(this.elementPrecision));
            iioMetadataNode.setAttribute("qtableId", Integer.toString(this.tableID));
            iioMetadataNode.setUserObject(new JPEGQTable(this.data));
            return iioMetadataNode;
        }
        
        void print() {
            System.out.println("Table id: " + Integer.toString(this.tableID));
            System.out.println("Element precision: " + Integer.toString(this.elementPrecision));
            new JPEGQTable(this.data).toString();
        }
    }
}
